#!/bin/sh
. /usr/share/libubox/jshn.sh

fwFile="/tmp/download"
NEW_FW_INFO="/tmp/new_fw_info"
check_times=3

server_path=$(uci get system.firmware.server)
version=$(cat /etc/version | grep Firmware | awk 'BEGIN{FS= " "} {print $4}')
model_U=$(uci get sysProductInfo.model.modelName)
model_L=$(uci get sysProductInfo.model.modelName| awk '{print tolower($0)}')

version_check()
{
	server_version=$1
	server_version_major=$(echo $server_version |cut -d  '.' -f 1)
	server_version_minor=$(echo $server_version |cut -d  '.' -f 2)
	server_version_release=$(echo $server_version |cut -d  '.' -f 3)
	server_version_build=$(echo $server_version |cut -d  '.' -f 4)

	version_major=$(echo $version |cut -d  '.' -f 1)
	version_minor=$(echo $version |cut -d  '.' -f 2)
	version_release=$(echo $version |cut -d  '.' -f 3)
	version_build=$(echo $version |cut -d  '.' -f 4)

	base_major=1000000000
	base_minor=1000000
	base_release=1000
	base_build=1

	now_version=$(($server_version_major*$base_major+$server_version_minor*$base_minor+$server_version_release*$base_release+$server_version_build*$base_build))
	version=$(($version_major*$base_major+$version_minor*$base_minor+$version_release*$base_release+$version_build*$base_build))

	echo "Server's Firmware Version: $server_version_major.$server_version_minor.$server_version_release.$server_version_build" > /dev/console
	echo "DUT's Firmware Version: $version_major.$version_minor.$version_release.$version_build" > /dev/console

	echo $now_version > /dev/console
	echo $version > /dev/console

	# return 1 => true, return 0 => false
	if [ $now_version -gt $version ]; then
		echo "check_firmware_version pass" > /dev/console
		echo 1
	else
	    echo "check_firmware_version fail" > /dev/console
	    echo 0
	fi

}

check_FW ()
{
	[ -f $NEW_FW_INFO ] && rm -rf $NEW_FW_INFO

	model="0"
	ver="0"
	file_size="0"
	release_date="0"
	md5sum="0"
	enable="0"
	last_check=$(date)

	result=$(curl -k -o $NEW_FW_INFO --connect-timeout 3 -m 10 "https://http.fw.updates1.netgear.com/${model_L}/${server_path}/fileinfo.txt" && echo "ok" || echo "fail")

	if [ "$result" == "ok" ];then
		ver=$(cat $NEW_FW_INFO |grep 'version'|cut -d '=' -f 2)
		verChkResult=$(version_check $ver)
		#echo "====verChkResult:$verChkResult" > /dev/console
		if [ $verChkResult == 1 ]; then
			enable="1"
		fi
		model=$(cat $NEW_FW_INFO |grep 'file'|cut -d '=' -f 2)
		file_size=$(cat $NEW_FW_INFO |grep 'size'|cut -d '=' -f 2)
		release_date=$(cat $NEW_FW_INFO |grep 'release'|cut -d '=' -f 2)
		md5sum=$(cat $NEW_FW_INFO |grep 'md5'|cut -d '=' -f 2)
	fi

	last_check=$(date)

	# generating new json data
	json_init
	json_add_string avaliable		"$enable"
	json_add_string model			"$model"
	json_add_string version			"$ver"
	json_add_string file_size		"$file_size"
	json_add_string release_date	"$release_date"
	json_add_string md5sum			"$md5sum"
	json_add_string last_check		"$last_check"

	MSG=`json_dump`
	echo "$MSG" > "$NEW_FW_INFO"
}

check() {
	count=1
	while [ $count -le $check_times ];do
		echo "==============check:$count time=============" >/dev/console
		check_FW

		avaliable=$(cat $NEW_FW_INFO | cut -d '"' -f 4)
		if [ "$avaliable" == "1" ]; then
			break
		else
			count=$(($count+1))
		fi
	done
}

download() {
	avaliable=$(cat $NEW_FW_INFO | cut -d '"' -f 4)
	filename=$(cat $NEW_FW_INFO |awk -F '"' '{print $8}')

	if [ "$avaliable" == "1" ]; then
		echo "==============download==start============" >/dev/console
		url="https://http.fw.updates1.netgear.com/${model_L}/${server_path}/${filename}"
		result=$(curl -k --connect-timeout 6 --speed-limit 1 --speed-time 5 -o $fwFile $url && echo 1 || echo 0)

		if [ "$result" == 1 ];then
			echo "==============download firmware succuess==========" >/dev/console
			echo 1
		else
			echo "==============download firmware fail===========" >/dev/console
			echo 0
		fi
	else
		echo "==============download firmware not avaliable=========" >/dev/console
		echo 0
	fi
}

upgrade() {
	[ -e "$fwFile" ] && {
		#sleep 60
		echo "==============upgrade==============" >/dev/console
		(sysupgrade -v $fwFile &) || (reboot)
	}
}


auto() {
	#do auto FW upgrade
	echo "==============auto==============" >/dev/console
	check
	sleep 2
	result=$(download)

	[ "$result" == 1 ] && upgrade
}

case $1 in
	download)		download;;
	upgrade)		upgrade;;
	check)			check;;
	auto)			auto;;
	*)				check;;
esac
